@@ -1,5 +1,5 @@
 ;; events.scm: DOM events exports and implementation for SDOM
-;; Copyright (C) 2004 Julian Graham
+;; Copyright (C) 2006 Julian Graham
 
 ;; SDOM is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
 ;; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 (define-module (sdom events)
-  #:use-module (ice-9 slib)
   #:use-module (srfi srfi-1)
   #:use-module (sdom core)
   #:export     (sdom:event?
@@ -26,21 +25,11 @@
 		sdom:add-event-listener!
 		sdom:remove-event-listener!
 		sdom:get-event-property
-		event-groups
-
-		lookup-event
 
 		sdom:dom-key-location-standard
 		sdom:dom-key-location-left
 		sdom:dom-key-location-right
-		sdom:dom-key-location-numpad
-
-		event-groups
-		
-		event-annotations
-		event-whole-annotation
-		event-annotation
-		event-annotate!))
+		sdom:dom-key-location-numpad))
 
 (sdom:register-feature! "Events" "3.0")
 (sdom:register-feature! "UIEvents" "3.0")
@@ -292,11 +281,23 @@
 			  groups)))
       (if handler #t #f))))      
 
+(define get-ancestors 
+  (lambda (node)
+    (if (null? node)
+	(list)
+	(let ((p (sdom:get-dom-property 
+		  node (if (eq? (car node) '@) 
+			   "sdom:owner-element" 
+			   "sdom:parent-node"))))
+	  (if (and p (not (null? p))) 
+	      (cons p (get-ancestors p))
+	      (list))))))
+
 (define sdom:will-trigger? 
   (lambda (node event-type . namespace-uri) 
     (if (find (lambda (x)
 		(sdom:has-event-listener? x event-type namespace-uri))
-	      (ancestors node))
+	      (get-ancestors node))
 	#t #f)))
 
 (define sdom:add-event-listener!
@@ -341,10 +342,8 @@
 	(throw 'sdom:exception sdom:exception-code-type-mismatch-err))
     (if (not (symbol? event))
 	(throw 'sdom:exception sdom:exception-code-type-mismatch-err))
-    
-    ;; (display "DISPATCH-EVENT: ") (display (symbol->string event)) (newline)
     (let* ((e `(*EVENT* ,event ,target))
-	   (a (append `(,target) (ancestors target)))
+	   (a (append `(,target) (get-ancestors target)))
 	   (proto (lookup-event event)))
       
       (event-annotate! e `(sdom:time-stamp ,(current-time)))
@@ -359,9 +358,6 @@
 			(event-annotate! 
 			 e `(,(car s) ,(apply (caaddr s) e data '()))))))
 		(cdddr proto))
-      ;; (display proto) (newline)
-
-
       (let* ((can-apply? (lambda (x p)
 			   (and (eq? (car x) event)
 				(or (eqv? p sdom:event-phase-at-target)
@@ -380,16 +376,10 @@
 						   (apply (caddr y) `(,e))))
 					     x))
 				 (map (lambda (x) (cdr x)) groups)))))))
-
-	;; (display "CAPTURE: ") (display (symbol->string event)) (newline)
 	(event-annotate! e `(sdom:event-phase ,sdom:event-phase-capturing))
 	(for-each (lambda (x) (f x sdom:event-phase-capturing)) a)
-	
-	;; (display "TARGET: ") (display (symbol->string event)) (newline)
 	(event-annotate! e `(sdom:event-phase ,sdom:event-phase-at-target))
 	(f (caddr e) sdom:event-phase-at-target)
-	
-	;; (display "BUBBLE: ") (display (symbol->string event)) (newline)
 	(event-annotate! e `(sdom:event-phase ,sdom:event-phase-bubbling))
 	(for-each (lambda (x) (f x sdom:event-phase-bubbling)) 
 		  (reverse a))
